/* NoSuch MIDI - Copyright 1997-1998 by Tim Thompson */

package com.nosuch.cooking;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import java.net.*;
import com.nosuch.midi.*;

public class Boomer extends Cooklet {

	BoomerState state;
	BoomerGui gui;
	BoomerPlayer player;

	public Boomer() {

		state = new BoomerState();
		state.setParent(this);
		// It's important that we make Player before Gui, because Gui
		// refers to Player things during initialization.
		player = new BoomerPlayer(this);
		gui = new BoomerGui(this);

	}

	public String getName() {
		return "Boomer";
	}

	public Dimension getPreferredSize() {
		return new Dimension(400,300);
	}

	public Panel getPanel() {
		return gui;
	}

	public void stop() {
		player.setRunning(false);
	}

	public void start() {
		// Don't start player or drawer running - that must be done explicitly
	}

	public void doLayout() {
		Rectangle r = getBounds();
		gui.setBounds(3,3,r.width-6,r.height-6);
	}

// 	public void readFromFile(String fname) {
// 		try {
// 			FileInputStream fis = new FileInputStream(fname);
// 			ObjectInputStream s = new ObjectInputStream(fis);
// System.out.println("AAA");
// 			state = (BoomerState)s.readObject();
// 			fis.close();
// 			gui.reInitialize();
// 		} catch (Exception ex) {
// 			ex.printStackTrace();
// 			System.out.println("Exception trying to read "+fname+" msg="+ex.getMessage());
// 		}
// 	}

	public void writeExternal(ObjectOutput out) throws IOException {
		state.writeExternal(out);
	}
	public void readExternal(ObjectInput in) throws IOException {
		state.readExternal(in);
System.out.println("REINITIALIZING GUI...");
		gui.reInitialize();
		gui.doLayout();
		gui.repaint();
	}
}

class BoomerPlayer {

	BoomerPlayThread player;
	BoomerDrawThread drawer;
	MidiPhrase[] phraseOn;
	Boomer parent;
	Random rand;

	BoomerPlayer(Boomer p) {
		parent = p;
		rand = new Random();
		// Create threads that loop continuously
		player = new BoomerPlayThread(parent);
		player.start();

		drawer = new BoomerDrawThread(parent);
		drawer.start();
	}

	void stop() {
		if ( player != null )
			player.setRunning(false);
		if ( drawer != null )
			drawer.setRunning(false);
	}

	void setRunning(boolean b) {
		player.setRunning(b);
		drawer.setRunning(b);
	}

	void enoughPhraseOn(int needed) {
		while ( phraseOn == null || phraseOn.length < needed ) {
			// Add to phraseOn array
			int oldsize;
			if ( phraseOn == null )
				oldsize = 0;
			else
				oldsize = phraseOn.length;
			MidiPhrase[] newphraseOn = new MidiPhrase[oldsize+1];
			if ( phraseOn != null )
				System.arraycopy(phraseOn,0,newphraseOn,0,oldsize);
			newphraseOn[oldsize] = new MidiPhrase();
			phraseOn = newphraseOn;
		}
	}

	synchronized void computestep(int sn) {
		enoughPhraseOn(sn+1);
		phraseOn[sn] = new MidiPhrase();
		for ( int dn=0; dn<parent.state.rows.size(); dn++ ) {
			DrumRow dr = (DrumRow)parent.state.rows.elementAt(dn);
			int v = dr.cells[sn];
			if ( v == 0 )
				continue;
			if ( v >= parent.state.cellLimit ) 
				phraseOn[sn].append(dr.note);
			else {
				int r = (rand.nextInt()%1000);
				if ( r < 0 )
					r = -r;
				if ( r < ((v*1000)/parent.state.cellLimit) )
					phraseOn[sn].append(dr.note);
			}
		}
	}

	void reComputeAll() {
		for ( int sn=0; sn<parent.state.nsteps; sn++ )
			computestep(sn);
	}

	MidiPhrase getPhrase() {
		reComputeAll();
		MidiPhrase ret = new MidiPhrase();
		long t = 0;
		for ( int n=0; n<parent.state.nsteps; n++ ) {
			MidiPhrase p = phraseOn[n];
			p.setLength(parent.state.stepsize);
			ret.append(p);
		}
		ret.setLength(parent.state.nsteps * parent.state.stepsize);
		return ret;
	}
}

class BoomerGui extends Panel implements ActionListener, MouseListener, ItemListener {

	Image imgScreen;
	int imgW, imgH;
	Graphics offScreen;
	Vector buttons;
	Vector xpos;
	Vector ypos;
	Button buttonMore;
	Button buttonStartStop;
	PopupMenu pm1;
	PopupMenu pm2;
	int labelWidth = 100;
	int yoff = 20;
	int xoff = labelWidth + 5;
	MyDrumListener drumListener;
	CheckboxMenuItem gradualOption;
	CheckboxMenuItem flashOption;
	Vector flashers;
	Boomer parent;

	public BoomerGui(Boomer b) {

		parent = b;

		pm2 = null;

		// One-time trivial stuff
		xpos = new Vector();
		buttons = new Vector();
		flashers = new Vector();
		ypos = new Vector();

		// Create the initial set of drum rows
		parent.state.setNrows(8);
		makeEnoughButtonsForRows(parent.state.rows.size());
		parent.state.changeDrum(0,"Bass Drum 1");
		parent.state.changeDrum(1,"Acoustic Snare");
		parent.state.changeDrum(2,"Closed Hi-Hat");
		parent.state.changeDrum(3,"Open Hi-Hat");
		parent.state.changeDrum(4,"Ride Cymbal 1");
		parent.state.changeDrum(5,"Hi Bongo");
		parent.state.changeDrum(6,"Low Bongo");
		parent.state.changeDrum(7,"Tambourine");
		reLabel();
		addSteps(16);

		// Create buttons at top
		buttonMore = new Button("More");
		buttonStartStop = new Button("Start");
		add("More",buttonMore);
		add("StartStop",buttonStartStop);
		buttonMore.addActionListener(this);
		buttonStartStop.addActionListener(this);

		pm1 = new PopupMenu("More1");

		Menu fm1 = new Menu("File");
		// fm1.add("Read File");
		// fm1.add("Write File");
		fm1.add("Read MIDI File");
		fm1.add("Write MIDI File");
		fm1.add("Write to Web");
		fm1.add("Write MIDI to Web");
		fm1.addActionListener(this);
		pm1.add(fm1);

		Menu rm1 = new Menu("Rows");
		rm1.add("Add 1 Row");
		rm1.add("Add 4 Rows");
		rm1.add("Add 8 Rows");
		rm1.add("Remove 1 Row");
		rm1.addActionListener(this);
		pm1.add(rm1);

		Menu sm1 = new Menu("Steps");
		sm1.add("Add 1 Step");
		sm1.add("Add 4 Steps");
		sm1.add("Add 8 Steps");
		sm1.add("Set to 4 Steps");
		sm1.add("Set to 16 Steps");
		sm1.add("Set to 32 Steps");
		sm1.add("Remove 1 Step");
		sm1.addActionListener(this);
		pm1.add(sm1);

		Menu om1 = new Menu("Option");
		// for some reason, checkboxmenuitems don't trigger
		// actions, so we need to use the ItemListener interface.
		om1.add(gradualOption=new CheckboxMenuItem("Gradual",parent.state.gradual));
		gradualOption.addItemListener(this);
		om1.add(flashOption=new CheckboxMenuItem("Flash",parent.state.flash));
		flashOption.addItemListener(this);
		pm1.add(om1);

		pm1.add("Clear All");
		add(pm1);	// add to Panel
		pm1.addActionListener(this);

		resetDrumMenu();
		setLayout(null);

		setBackground(Color.lightGray);

		repaint();

		addMouseListener(this);
	}

	public void resetDrumMenu() {
		if ( pm2 != null )
			remove(pm2);
		pm2 = new PopupMenu("More2");
		drumListener = new MyDrumListener();
		pm2.add(drumSubMenu(drumListener,"basic"));
		pm2.add(drumSubMenu(drumListener,"cymbal"));
		pm2.add(drumSubMenu(drumListener,"tom"));
		pm2.add(drumSubMenu(drumListener,"latin"));
		pm2.add(drumSubMenu(drumListener,"other"));
		pm2.add("Clear Row");
		pm2.addActionListener(drumListener);
		add(pm2);	// add to Panel
	}

	public void removeStep() {
		if ( parent.state.nsteps > 1 ) {
			parent.state.setNsteps(parent.state.nsteps-1);
			makeEnoughGUIForSteps(parent.state.nsteps);
		}
	}

	public void addSteps(int n) {
		parent.state.setNsteps(parent.state.nsteps+n);
		// We could just call reInitialize() here.
		makeEnoughGUIForSteps(parent.state.nsteps);
		parent.player.reComputeAll();
	}

	void reInitialize() {
		makeEnoughGUIForSteps(parent.state.nsteps);
		makeEnoughButtonsForRows(parent.state.rows.size());
		reLabel();
		parent.player.reComputeAll();
		gradualOption.setState(parent.state.gradual);
		flashOption.setState(parent.state.flash);
	}

	// make sure button label agrees with drum row
	void reLabelButton(int num) {
		DrumRow dr = (DrumRow)(parent.state.rows.elementAt(num));
		Button b = (Button)buttons.elementAt(num);
		b.setLabel(dr.label);
	}

	void reLabel() {
		for ( int n=0; n<parent.state.rows.size(); n++ )
			reLabelButton(n);
	}

	public CellInfo cellAtPoint(int x, int y) {
		int y0 = ((Integer)ypos.elementAt(0)).intValue();
		if ( y < y0 )
			return null;
		for ( int r=1; r<=parent.state.rows.size(); r++ ) {
			y0 = ((Integer)ypos.elementAt(r)).intValue();
			if ( y < y0 ) {
				// It's in row r-1, now find column
				int x0 = ((Integer)xpos.elementAt(0)).intValue();
				if ( x < x0 )
					return null;
				for ( int c=1; c<=parent.state.nsteps; c++ ) {
					x0 = ((Integer)xpos.elementAt(c)).intValue();
					if ( x < x0 )
						return parent.state.getCell(r-1,c-1);
				}
			}
		}
		return null;
				
	}

	public synchronized void doLayout() {

		// We need to resize the offscreen bitmap
		initOffScreen();

		Rectangle r = getBounds();
		setBounds(r.x,r.y,r.width,r.height);

		ypos.removeAllElements();
		xpos.removeAllElements();
		int dx = (xoff-2)/2;
		buttonStartStop.setBounds(0,1,dx-1,yoff-1);
		buttonMore.setBounds(dx+1,1,dx,yoff-1);

		// Loop through rows, and (if necessary) layout buttons, etc.
		int yheight = r.height - yoff;
		int oldy = yoff;
		ypos.addElement(new Integer(oldy));

		for ( int n=0; n<parent.state.rows.size(); n++ ) {

			int y = r.height - 1 - ((parent.state.rows.size()-1-n)*yheight)/parent.state.rows.size();
			Button b = (Button)buttons.elementAt(n);
			b.setBounds(0,oldy+1,xoff-2,y-oldy);

			oldy = y;
			ypos.addElement(new Integer(oldy));
		}
		int xwidth = r.width - xoff;
		int y0 = ((Integer)ypos.elementAt(0)).intValue();
		int y1 = ((Integer)ypos.elementAt(parent.state.rows.size())).intValue();
		int x;
		for ( int n=0; n<parent.state.nsteps; n++ ) {
			if ( n == 0 )
				x = xoff;
			else
				x = r.width - ((parent.state.nsteps-n)*xwidth)/parent.state.nsteps;
			xpos.addElement(new Integer(x));
		}
		x = r.width - 1;
		xpos.addElement(new Integer(x));

		// Lay out the flashers at the top
		y0 = ((Integer)ypos.elementAt(0)).intValue();
		for ( int n=0; n<parent.state.nsteps; n++ ) {
		 	int xa = ((Integer)xpos.elementAt(n)).intValue();
		 	int xb = ((Integer)xpos.elementAt(n+1)).intValue();
			Label p = (Label)flashers.elementAt(n);
			p.setBounds(xa+1,1,xb-xa-1,y0-2);
		}
		drawAllOffScreen();
		repaint();
	}

	public boolean getFlashing() {
		return flashOption.getState();
	}

	public Menu drumSubMenu(MyDrumListener mdl, String cat){
		Menu m = new Menu(cat);
		m.addActionListener(mdl);
		for ( Enumeration e = parent.state.drums.elements(); e.hasMoreElements(); ) {
			DrumInfo di = (DrumInfo)e.nextElement();
			if ( di.category.equals(cat) ) {
				m.add(di.label);
			}
		}
		return m;
	}

	public void initOffScreen() {
		Dimension d = getSize();
		imgScreen = createImage(d.width,d.height);
		imgW = d.width;
		imgH = d.height;
		offScreen = imgScreen.getGraphics();
		// offScreen.setColor(Color.black);
		// offScreen.drawRect(0,0,d.width-1,d.height-1);
	}

	public synchronized void flashon(int n) {
		Label p = (Label)flashers.elementAt(n);
		p.setBackground(Color.green);
	}
	public synchronized void flashoff(int n) {
		Label p = (Label)flashers.elementAt(n);
		p.setBackground(Color.lightGray);
	}

	void makeEnoughButtonsForRows(int needed) {

		while ( buttons.size() < needed ) {
			int num = buttons.size();
			DrumInfo di = (DrumInfo)(parent.state.drums.elementAt(0));
			Button b = new Button(di.label);
			buttons.addElement(b);
			b.addActionListener(new MyButtonListener(num,this));
			add("b"+num,b);	// add button to BoomerGui
		}
	}

	void makeEnoughGUIForSteps(int needed) {

		// Make sure flashers are big enough

		while ( flashers.size() < needed ) {
			Label f = new Label(Integer.toString(flashers.size()+1),Label.CENTER);
			flashers.addElement(f);
			f.setBackground(Color.lightGray);
			add("f"+Integer.toString(flashers.size()),f);
		}
	}

	public void itemStateChanged(ItemEvent e) {
		if ( e.getItemSelectable() == gradualOption ) {
			parent.state.gradual = gradualOption.getState();
		}
		else if ( e.getItemSelectable() == flashOption ) {
			parent.state.flash = flashOption.getState();
		}
	}

	public void setNrows(int n) {
		parent.state.setNrows(n);
		makeEnoughButtonsForRows(parent.state.rows.size());
	}

	int getNrows() {
		return parent.state.getNrows();
	}

	public void actionPerformed(ActionEvent e) {
		String a = e.getActionCommand();
		boolean layout = false;
		if ( a.equals("More") ) {
			Rectangle r = buttonMore.getBounds();
			pm1.show(this,r.x,r.y + r.height);
		}
		else if ( a.equals("Clear All") ) {
			for ( int rn=0; rn<parent.state.rows.size(); rn++ ) {
				parent.state.clearRow(rn);
				rowDraw(rn);
			}
			repaint();
		}
		else if ( a.equals("Add 1 Step") ) {
			addSteps(1);
			layout = true;
		}
		else if ( a.equals("Remove 1 Step") ) {
			removeStep();
			layout = true;
		}
		else if ( a.equals("Add 4 Steps") ) {
			addSteps(4);
			layout = true;
		}
		else if ( a.equals("Add 8 Steps") ) {
			addSteps(8);
			layout = true;
		}
		else if ( a.equals("Set to 4 Steps") ) {
			parent.state.setNsteps(4);
			reInitialize();
			layout = true;
		}
		else if ( a.equals("Set to 16 Steps") ) {
			parent.state.setNsteps(16);
			reInitialize();
			layout = true;
		}
		else if ( a.equals("Set to 32 Steps") ) {
			parent.state.setNsteps(32);
			reInitialize();
			layout = true;
		}
		else if ( a.equals("Remove 1 Row") ) {
			if ( getNrows() > 1 ) {
				setNrows(getNrows()-1);
				layout = true;
			}
		}
		else if ( a.equals("Add 1 Row") ) {
			setNrows(getNrows()+1);
			layout = true;
		}
		else if ( a.equals("Add 4 Rows") ) {
			setNrows(getNrows()+4);
			layout = true;
		}
		else if ( a.equals("Add 8 Rows") ) {
			setNrows(getNrows()+8);
			layout = true;
		}
// 		else if ( a.equals("Write File") ) {
// 			FileDialog fd = new FileDialog(new Frame(),"Write Boomer File",FileDialog.SAVE);
// 			fd.show();
// 			String f = fd.getFile();
// 			String d = fd.getDirectory();
// 			try {
// 				FileOutputStream fos = new FileOutputStream(d+f);
// 				ObjectOutputStream s = new ObjectOutputStream(fos);
// 				s.writeObject(parent.state);
// 				s.flush();
// 				fos.close();
// 			} catch (Exception ex) {
// 				System.out.println("Exception trying to open "+f+" msg="+ex.getMessage());
// 			}
// 		}
		else if ( a.equals("Write MIDI File") ) {
			FileDialog fd = new FileDialog(new Frame(),"Write Boomer MIDI File",FileDialog.SAVE);
			fd.show();
			String f = fd.getFile();
			String d = fd.getDirectory();
			try {
				FileOutputStream fos = new FileOutputStream(d+f);
				MidiFileWriter mf = new MidiFileWriter(fos);
				mf.addTrack(parent.player.getPhrase());
				mf.write();
				fos.close();
			} catch (Exception ex) {
				System.out.println("Exception trying to open "+f+" msg="+ex.getMessage());
			}
		}
		else if ( a.equals("Write to Web") ) {
			AskNameDialog nd = new AskNameDialog(new Frame());
			nd.show();
			if ( nd.wasCancelled() )
				return;

			String nm = nd.getUserName();
			String pat = nd.getPatternName();
			String email = nd.getEmail();

			// Get the MIDI file into a byte array
			ByteArrayOutputStream bo = new ByteArrayOutputStream();
			MidiFileWriter mf = new MidiFileWriter(bo);
			mf.addTrack(parent.player.getPhrase());
			try {
				mf.write();
				bo.flush();
			} catch (IOException ioe) {
				System.out.println("Exception tryint to produce MIDI file?");
			}
			byte[] mm = bo.toByteArray();

			writeToWeb(nm,pat,parent.state,mm,email);
		}
// 		else if ( a.equals("Read File") ) {
// 			FileDialog fd = new FileDialog(new Frame(),"Read Boomer File",FileDialog.LOAD);
// 			fd.show();
// 			String f = fd.getFile();
// 			String d = fd.getDirectory();
// 			try {
// 				FileInputStream fis = new FileInputStream(d+f);
// 				ObjectInputStream s = new ObjectInputStream(fis);
// 
// 				BoomerState nbs = new BoomerState();
// 
// 				parent.state = (BoomerState)s.readObject();
// 				parent.state.setParent(parent);
// 
// 				fis.close();
// 				reInitialize();
// 				layout = true;
// 			} catch (Exception ex) {
// 				ex.printStackTrace();
// 				System.out.println("Exception trying to read "+f+" msg="+ex.getMessage());
// 			}
// 		}
		else if ( a.equals("Read MIDI File") ) {
			FileDialog fd = new FileDialog(new Frame(),"Read Boomer File",FileDialog.LOAD);
			fd.show();
			String f = fd.getFile();
			String d = fd.getDirectory();
			if ( f == null || d == null )	// user cancelled
				return;
			try {
				MidiPhrase p = MidiFileReader.readMidiPhrase(d+f);
				parent.state = new BoomerState();
				parent.state.setParent(parent);
				parent.state.initFromPhrase(p);
				reInitialize();
				layout = true;
			} catch (Exception ex) {
				ex.printStackTrace();
				System.out.println("Exception trying to read "+f+" msg="+ex.getMessage());
			}
		}
		else if ( a.equals("Start") ) {
			buttonStartStop.setLabel("Stop");
			parent.player.setRunning(true);
		}
		else if ( a.equals("Stop") ) {
			buttonStartStop.setLabel("Start");
			parent.player.setRunning(false);
		}
		if ( layout )
			doLayout();
	}

	private void writeToWeb(String nm, String pat, Object obj, byte[] mm, String email) {
		KitchenPost k;
		boolean stat;
		try {
			String proxy = null;
			int port = 8000;

			// k = new KitchenPost(proxy,port,"www.nosuch.com",80,"/cook/protected/cookpost.cgi");
			k = new KitchenPost("gate2.geoplex.com",8000,"www.nosuch.com",80,"/cook/protected/cookpost.cgi");

System.out.println("writeToWeb after KitchenPost new");
		} catch (IOException e2) {
			System.out.println("Exception trying to connect to Kitchen Host msg="+e2.getMessage());
			return;
		}

		try {
			stat = k.postIt(nm,pat,parent.state,mm,email,parent.state.nsteps);
		}
		catch (IOException e3) {
			e3.printStackTrace();
			System.out.println("Exception trying to save object msg="+e3.getMessage());
			stat = false;
		}
		System.out.println("stat = "+stat);
	}

	class MyDrumListener implements ActionListener {
		int rownum;
		MyDrumListener() {
			rownum = -1;
		}
		void setDrumRowNum(int n) {
			rownum = n;
		}
		public void actionPerformed(ActionEvent e) {
			String a = e.getActionCommand();
			if ( rownum < 0 )
				System.out.println("Error - dr==null in MyDrumListener");
			else {
				if ( a.equals("Clear Row") ) {
					parent.state.clearRow(rownum);
					rowDraw(rownum);
					repaint();
				}
				else {
					Button b = (Button)buttons.elementAt(rownum);
					b.setLabel(a);
					parent.state.changeDrum(rownum,a);
				}
				parent.player.reComputeAll();
			}
		}
	}

	class MyButtonListener implements ActionListener {
		int num;
		Panel panel;
		MyButtonListener(int n, Panel p) {
			num = n;
			panel = p;
		}
		public void actionPerformed(ActionEvent e) {
			String a = e.getActionCommand();
			int y = ((Integer)ypos.elementAt(num+1)).intValue();
			// The drumListener is the listener for the pop-up menu.
			drumListener.setDrumRowNum(num);
			// show the drum pop-up menu
			pm2.show(panel,0,y);
		}
	}

	public DrumRow newDrumRow(DrumInfo di) {
		makeEnoughButtonsForRows(buttons.size()+1);
		DrumRow dr = new DrumRow(di.label, di.note, parent.state.nsteps);
		((Button)buttons.elementAt(buttons.size()-1)).setLabel(di.label);
		return dr;
	}

	public void rowDraw(int r) {
		for ( int c=0; c<parent.state.nsteps; c++ )
			cellDraw(parent.state.getCell(r,c));
	}

	public synchronized void cellDraw(CellInfo ci) {
		int x0 = ((Integer)xpos.elementAt(ci.col)).intValue();
		x0++;
		int y0 = ((Integer)ypos.elementAt(ci.row)).intValue();
		y0++;
		int x1 = ((Integer)xpos.elementAt(ci.col+1)).intValue();
		x1--;
		int y1 = ((Integer)ypos.elementAt(ci.row+1)).intValue();
		// y1--;
		int v = ci.value();
		if ( v == 0 ) {
			offScreen.setColor(Color.lightGray);
			offScreen.fillRect(x0,y0,x1-x0,y1-y0);
		}
		else {
			offScreen.setColor(Color.green);
			int dy = (v * (y1 - y0)) / parent.state.cellLimit;
			offScreen.fillRect(x0,y1-dy,x1-x0,dy);
		}
	}

	public void update(Graphics g) {
		paint(g);
	}

	public synchronized void drawAllOffScreen() {

		if ( offScreen == null )
			initOffScreen();

		Rectangle b = getBounds();

		offScreen.setColor(Color.lightGray);
		offScreen.fillRect(0,0,imgW,imgH);

		offScreen.setColor(Color.black);

		// Draw the grid lines
		offScreen.drawLine(xoff,0,b.width,0);
		for ( int n=0; n<=parent.state.rows.size(); n++ ) {
			int y = ((Integer)ypos.elementAt(n)).intValue();
			offScreen.drawLine(xoff,y,b.width,y);
		}
		int y0 = ((Integer)ypos.elementAt(0)).intValue();
		int y1 = ((Integer)ypos.elementAt(parent.state.rows.size())).intValue();
		for ( int n=0; n<=parent.state.nsteps; n++ ) {
			int x = ((Integer)xpos.elementAt(n)).intValue();
			if ( (n%4) == 0 )
				offScreen.drawLine(x,b.y,x,y1);
			else
				offScreen.drawLine(x,y0,x,y1);
		}

		// for ( int n=0; n<parent.state.nsteps; n++ )
		// 	flashon(n);

		// Draw the cells
		for ( int r=0; r<parent.state.rows.size(); r++ )
			rowDraw(r);
	}

	public synchronized void paint(Graphics g) {
		if ( offScreen == null )
			drawAllOffScreen();
		g.drawImage(imgScreen, 0, 0, this);
	}


	public void mouseClicked(MouseEvent e) { };
	public void mouseEntered(MouseEvent e) { };
	public void mouseExited(MouseEvent e) { };
	public void mousePressed(MouseEvent e) {
		CellInfo ci;
		ci = parent.gui.cellAtPoint(e.getX(),e.getY());
		if ( ci != null ) {
			if ( gradualOption.getState() )
				parent.state.incrementCell(ci.row,ci.col);
			else
				parent.state.toggleCell(ci.row,ci.col);
			cellDraw(ci);
			repaint();
			parent.player.computestep(ci.col);
		}
	}
	public void mouseReleased(MouseEvent e) { };

}

class AskNameDialog extends Dialog {
	String _name;
	String _patt;
	String _email;
	TextField nametext;
	Label namelabel;
	TextField patttext;
	Label pattlabel;
	TextField emailtext;
	Label emaillabel;
	Button butt1, butt2;
	boolean _cancelled;
	public AskNameDialog(Frame f) {

		super(f,"Enter Name",true);

		_cancelled = false;
		setLayout(null);

		add(pattlabel = new Label("The Pattern Name:",Label.RIGHT));
		add(patttext = new TextField());

		add(namelabel = new Label("Your Name:",Label.RIGHT));
		add(nametext = new TextField());
		String s = KitchenData.getUserName();
		if ( s != null )
			nametext.setText(s);

		add(emaillabel = new Label("Your Email Address:",Label.RIGHT));
		add(emailtext = new TextField());
		s = KitchenData.getEmail();
		if ( s != null )
			emailtext.setText(s);

		add(butt1 = new Button("OK"));
		butt1.addActionListener(new okAction());

		add(butt2 = new Button("Cancel"));
		butt2.addActionListener(new cancelAction());

		setSize(new Dimension(300,200));

		patttext.requestFocus();
	}

	public boolean wasCancelled() {
		return _cancelled;
	}
	public String getUserName() {
		return _name;
	}
	public String getPatternName() {
		return _patt;
	}
	public String getEmail() {
		return _email;
	}
	public void doLayout() {
		Rectangle r = getBounds();
		int w = r.width;
		int h = r.height;

		int x0 = 3;
		int y0 = 30;
		int xm = w/2;
		int x1 = w;
		int y1 = h;

		int dy = 30;
		int y = y0;
		pattlabel.setBounds(x0,y,xm-5,dy-4);
		patttext.setBounds(xm+2,y,x1-xm-20,dy-4);

		y += dy;
		namelabel.setBounds(x0,y,xm-5,dy-4);
		nametext.setBounds(xm+2,y,x1-xm-20,dy-4);

		y += dy;
		emaillabel.setBounds(x0,y,xm-5,dy-4);
		emailtext.setBounds(xm+2,y,x1-xm-20,dy-4);

		butt1.setBounds(w/4,y1-dy-4,w/4-2,dy-2);
		butt2.setBounds(w/2+2,y1-dy-4,w/4-2,dy-2);
	}

	class okAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
	
			_name = nametext.getText();
			_patt = patttext.getText();
			_email = emailtext.getText();
	
			KitchenData.setUserName(_name);
			KitchenData.setEmail(_email);
	
			dispose();
		}
	}
	class cancelAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			_cancelled = true;
			dispose();
		}
	}
}

class BoomerPlayThread extends Thread {

	int currentStep;
	Integer pauseObject;
	boolean isRunning;
	Boomer parent;

	BoomerPlayThread(Boomer p) {
		pauseObject = new Integer(0);
		isRunning = false;
		currentStep = -1;
		parent = p;
	}

	public void setRunning(boolean b) {
		isRunning = b;
		synchronized (pauseObject) {
			pauseObject.notify();
		}
	}

	public void run() {
		Thread t = Thread.currentThread();

		long tm = MidiSystem.now();
		while ( true ) {
			if ( isRunning ) {

				// Don't save the results of milliToClick, because
				// tempo will change while we're running.
				long nextfire = tm + parent.state.stepsize;

				if ( ++currentStep >= parent.state.nsteps ) {
					currentStep = 0;
				}
				MidiPhrase p = parent.player.phraseOn[currentStep];
				if ( p != null ) {
					try {
						MidiSystem.schedule(p,nextfire);
					} catch (Exception e) {
					}
				}

				if ( currentStep == 0 ) {
					parent.player.reComputeAll();	// this is for "gradual"
											// cells that need recomputing
				}

				long dt = nextfire - MidiSystem.now();

				if ( dt > 0 ) {
					try {
						long ml = MidiSystem.clickToMilli(dt);
						t.sleep(ml);
					} catch (Exception e) {
						System.out.println("exception while sleeping = "+e);
					}
				}

				tm = nextfire;
			}
			else {
				synchronized (pauseObject) {
					try {
						pauseObject.wait();
					} catch (Exception e) {
						System.out.println("exception while waiting on pauseObject = "+e);
					}
				}
				tm = Alg.nextquant(MidiSystem.now(),parent.state.startQuant);
				// always start at step 0
				currentStep = 0;
				MidiPhrase p = parent.player.phraseOn[currentStep];
				try {
					MidiSystem.schedule(p,tm);
				} catch (Exception e) {
				}
			}
		}
	}
}

class BoomerDrawThread extends Thread {

	int currentStep;
	Integer pauseObject;
	boolean isRunning;
	Boomer parent;

	BoomerDrawThread(Boomer p) {
		parent = p;
		pauseObject = new Integer(0);
		isRunning = false;
		currentStep = -1;
	}

	public void setRunning(boolean b) {
		// don't turn on if it's off and we don't want it turned on
		if ( parent.gui.getFlashing() == false && isRunning == false )
			return;
		isRunning = b;
		synchronized (pauseObject) {
			pauseObject.notify();
		}
	}

	public void run() {
		Thread t = Thread.currentThread();

		long tm = MidiSystem.now();
		while ( true ) {
			if ( isRunning ) {

				// Don't save the results of milliToClick, because
				// tempo will change while we're running.
				long nextfire = tm + parent.state.stepsize;
				long dt = nextfire - MidiSystem.now();

				if ( dt > 0 ) {
					try {
						t.sleep(MidiSystem.clickToMilli(dt));
					} catch (Exception e) {
						System.out.println("exception while sleeping = "+e);
					}
				}
				tm = nextfire;

				if ( currentStep < 0 )
					currentStep = 0;
				else {
					parent.gui.flashoff(currentStep);
					// we depend on the repaint being done below.
					if ( ++currentStep >= parent.state.nsteps )
						currentStep = 0;
				}
				parent.gui.flashon(currentStep);
				parent.gui.repaint();
			}
			else {
				if ( currentStep >= 0 ) {
					parent.gui.flashoff(currentStep);
					parent.gui.repaint();
				}
				currentStep = -1;
				synchronized (pauseObject) {
					try {
						pauseObject.wait();
					} catch (Exception e) {
						System.out.println("exception while waiting on pauseObject = "+e);
					}
				}
				tm = Alg.nextquant(MidiSystem.now(),parent.state.startQuant);

				// sleep till the time arrives, so the initial
				// flash is on the beat.
				long dt = tm - MidiSystem.now();
				if ( dt > 0 ) {
					try {
						t.sleep(MidiSystem.clickToMilli(dt));
					} catch (Exception e) {
						System.out.println("exception while sleeping = "+e);
					}
				}
				// always start at step 0
				currentStep = 0;
				parent.gui.flashon(currentStep);
				parent.gui.repaint();
			}
		}
	}
}

class KitchenPost {

	Socket _s;
	String _path;
	String proxy;
	int proxyport;
	String host;
	int port;

	public KitchenPost(String pr, int pp, String h,int prt,String path) throws IOException {
		proxy = pr;
		proxyport = pp;
		host = h;
		port = prt;
		if ( proxy != null ) {
			try {
				_s = new Socket(proxy,proxyport);
System.out.println("after new Socket proxy="+proxy+" proxyport="+proxyport);
			} catch (IOException e) {
				System.out.println("Unable to connect to "+host+":"+port);
				return;
			}
		}
		else {
			try {
				_s = new Socket(host,port);
System.out.println("after new Socket host="+host+" port="+port);
			} catch (IOException e) {
				System.out.println("Unable to connect to "+host+":"+port);
				_s = null;
				return;
			}
		}
		_path = path;
	}

	public boolean postIt(String nm, String pat, Object obj, byte[] mm, String email, int nsteps) throws IOException {

		if ( _s == null )
			return false;

		System.out.println("writeObject start");

		OutputStream o = _s.getOutputStream();
		DataOutputStream dout = new DataOutputStream(o);

		InputStream i = _s.getInputStream();
        BufferedReader di = new BufferedReader(new InputStreamReader(i));

		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		ObjectOutputStream s = new ObjectOutputStream(bo);

		Externalizable eobj = (Externalizable)obj;
		eobj.writeExternal(s);

		s.flush();
		byte[] bb = bo.toByteArray();

// int fct = 2;
// System.out.println("bb is being cut by "+fct);
// bb = new byte[bb.length/fct];

System.out.println("bb size = "+bb.length);

		// This is the first line in the data we send
		String line1 = "nm="+nm
				+"&pt="+pat
				+"&em="+email
				+"&ml="+mm.length
				+"&ol="+bb.length
				+"&st="+nsteps
				+"\n";

		System.out.println("line1="+line1);
		System.out.println("writeObject bb.length="+bb.length
			+"  bb0 is "+(int)(bb[0]) +"  bb1 is "+(int)(bb[1]));

		// String args = "?midilength="+mm.length+"&objectlength="+bb.length;
		String args = "";
		String ss;
		String EOL = "\r\n";
		if ( proxy != null )
			ss = "POST http://"+host+":"+port+_path+args+" HTTP/1.0"+EOL;
		else
			ss = "POST "+_path+args+" HTTP/1.0"+EOL;
System.out.println("ss = "+ss);

		dout.writeBytes(ss);
		boolean sendboth = true;
		String cl;
		if ( sendboth == true )
			cl = "Content-Length: "+(mm.length+bb.length+line1.length())+EOL;
		else
			cl = "Content-Length: "+(mm.length+line1.length())+EOL;
System.out.println("cl = "+cl);
		dout.writeBytes(cl);
		dout.flush();
		dout.writeBytes(EOL);
		dout.writeBytes(line1);
		dout.flush();
		dout.write(mm);
		dout.flush();
		dout.write(bb);
		// if ( sendboth == true ) {
		// 	sendslow(dout,bb);
		// }
		dout.flush();
		
		String st;
System.out.println("trying to read");
		while ( (st=di.readLine()) != null ) {
			System.out.println("READLINE st="+st);
		}
		di.close();
		return true;
	}
	void sendslow(DataOutputStream out,byte[] bb) throws IOException {
		int sofar = 0;
		while ( sofar < bb.length ) {
			int thistime = 200;
			if ( (sofar+thistime) > bb.length )
				thistime = bb.length - sofar;
System.out.println("Writing sofar="+sofar+" thistime="+thistime);
			out.write(bb,sofar,thistime);
			out.flush();
			try {
			Thread.currentThread().sleep(1000);
			} catch (Exception e) {}
			sofar += thistime;
		}
System.out.println("done");
	}
}
