/* NoSuch MIDI - Copyright 1997 by Tim Thompson */

package com.nosuch.cooking;

import com.nosuch.midi.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * DrumMap is a class encapsulating an ordered list of drums - meant for
 * constructing menus and things.
 */

public class DrumMap {

	Vector drums;

	class Drum {
		String name;
		MidiPhrase drum;
		String category;
		Drum(MidiPhrase d, String n, String cat) {
			drum = d;
			name = n;
			category = cat;
		}
		Drum(String d, String n, String cat) {
			drum = new MidiPhrase(d);
			name = n;
			category = cat;
		}
	}

	public DrumMap() {
		drums = new Vector();
		addElement("p35c10","Ac. Bass Drum",		"basic");
		addElement("p36c10","Bass Drum 1",			"basic");
		addElement("p37c10","Side Stick",			"basic");
		addElement("p38c10","Acoustic Snare",		"basic");
		addElement("p39c10","Hand Clap",			"basic");
		addElement("p40c10","Electric Snare",		"snare");
		addElement("p41c10","Low Floor Tom",		"tom");
		addElement("p42c10","Closed Hi-Hat",		"cymbal");
		addElement("p43c10","High Floor Tom",		"tom");
		addElement("p44c10","Pedal Hi-Hat",			"cymbal");
		addElement("p45c10","Low Tom",				"tom");
		addElement("p46c10","Open Hi-Hat",			"cymbal");
		addElement("p47c10","Low-Mid Tom",			"tom");
		addElement("p48c10","Hi-Mid Tom",			"tom");
		addElement("p49c10","Crash Cymbal 1",		"cymbal");
		addElement("p50c10","High Tom",				"tom");
		addElement("p51c10","Ride Cymbal 1",		"cymbal");
		addElement("p52c10","Chinese Cymbal",		"cymbal");
		addElement("p53c10","Ride Bell",			"cymbal");
		addElement("p54c10","Tambourine",			"basic");
		addElement("p55c10","Splash Cymbal",		"cymbal");
		addElement("p56c10","Cowbell",				"basic");
		addElement("p57c10","Crash Cymbal 2",		"cymbal");
		addElement("p58c10","Vibraslap",			"other");
		addElement("p59c10","Ride Cymbal 2",		"cymbal");
		addElement("p60c10","Hi Bongo",				"latin");
		addElement("p61c10","Low Bongo",			"latin");
		addElement("p62c10","Mute Hi Conga",		"latin");
		addElement("p63c10","Open Hi Conga",		"latin");
		addElement("p64c10","Low Conga",			"latin");
		addElement("p65c10","High Timbale",			"latin");
		addElement("p66c10","Low Timbale",			"latin");
		addElement("p67c10","High Agogo",			"latin");
		addElement("p68c10","Low Agogo",			"latin");
		addElement("p69c10","Cabasa",				"latin");
		addElement("p70c10","Maracas",				"latin");
		addElement("p71c10","Short Whistle",		"other");
		addElement("p72c10","Long Whistle",			"other");
		addElement("p73c10","Short Guiro",			"other");
		addElement("p74c10","Long Guiro",			"other");
		addElement("p75c10","Claves",				"other");
		addElement("p76c10","Hi Wood Block",		"other");
		addElement("p77c10","Low Wood Block",		"other");
		addElement("p78c10","Mute Cuica",			"other");
		addElement("p79c10","Open Cuica",			"other");
		addElement("p80c10","Mute Triangle",		"other");
		addElement("p81c10","Open Triangle",		"other");
	}

	private void addElement(String s1, String nm, String ct) {
		drums.addElement(new Drum(s1,nm,ct));
	}

	public void addElement(MidiPhrase p, String nm, String ct) {
		drums.addElement(new Drum(p,nm,ct));
	}

	public int size() {
		return drums.size();
	}
	public String getName(int n) {
		Object o = drums.elementAt(n);
		if ( o == null )
			return null;
		return ((Drum)o).name;
	}
	public String getNameOf(int pitch, int chan) {
		for ( Enumeration e=drums.elements(); e.hasMoreElements(); ) {
			Drum d = (Drum)e.nextElement();
			MidiMsg m = d.drum.getAt(0);
			if ( m.getPitch() == pitch && m.getChannel() == chan )
				return d.name;
		}
		return null;
	}
	public MidiPhrase getPhrase(int n) {
		Object o = drums.elementAt(n);
		if ( o == null )
			return null;
		return ((Drum)o).drum;
	}
	public String getCategory(int n) {
		Object o = drums.elementAt(n);
		if ( o == null )
			return null;
		return ((Drum)o).category;
	}
}
