/* NoSuch MIDI - Copyright 1997-1998 by Tim Thompson */

package com.nosuch.cooking;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.beans.*;
import java.applet.*;
import com.nosuch.midi.*;

public class Kitchen {

	static Frame f;
	public static String root;

	public static void main(String args[]) {

		root = System.getProperty("kitchen");

		if ( root == null || root.equals("") ) {
			// System.err.println("No value for kitchen, assuming c:\\tmp");
			root = "c:\\tmp";
		}

		boolean ok = MidiSystem.start(null,-1,0);
		if ( ! ok ) {
			System.err.println("Can't initialize MIDI!?");
			// System.exit(1);
		}

		f = new KitchenFrame("Kitchen");
		// f = new Frame("Kitchen");

		f.setLayout(new BorderLayout(5,5));

		Panel p1 = mkpanel("com.nosuch.cooking.Volume","Volume");
		Panel p2 = mkpanel("com.nosuch.cooking.Tempo","Tempo");

		// f.add("North",new Label("Cooking with MIDI",Label.CENTER));
		f.add("East",p1);
		f.add("West",p2);
		f.setSize(new Dimension(120, 200));
		f.show();
		f.addWindowListener(new TopWindowListener());

		if ( !ok )
			System.out.println("MidiSystem NOT started");

		// KitchenServer t = new KitchenServer();
		// t.start();

		// createCooklet("com.nosuch.cooking.Boomer",null);
	}

	static public Panel mkpanel(String kit, String label) {
		Cooklet c1 = rawCooklet(kit);
		Panel p = new Panel();
		p.setLayout(new BorderLayout(5,5));
		p.add("North",new Label(label,Label.CENTER));
		p.add("Center",c1);
		return p;
	}

	static public Cooklet rawCooklet(String className) {
		Class c;
		try {
			c = Class.forName(className);
		} catch (Exception e) {
			System.out.println("Unable to find class "+className+" msg="+e.getMessage());
			return null;
		}
		Cooklet j;
		try {
			j = (Cooklet)c.newInstance();
		} catch (Exception e) {
			System.out.println("Unable to make instance of "+className+" msg="+e.getMessage());
			return null;
		}
		j.init();
		j.start();
		return j;
	}

	static public Cooklet createCooklet(String className, InputStream in) {
		Class c;
		try {
			c = Class.forName(className);
		} catch (Exception e) {
			System.out.println("Unable to find class "+className+" msg="+e.getMessage());
			return null;
		}
		Cooklet j;
		try {
			j = (Cooklet)c.newInstance();
		} catch (Exception e) {
			System.out.println("Unable to make instance of "+className+" msg="+e.getMessage());
			return null;
		}

		Frame f = new Frame(j.getName());
		j.init();
		j.start();
		f.add("Center",j);
		f.setSize(j.getPreferredSize());
		f.show();
		f.addWindowListener(new CookletWindowListener(f,j));
		f.addComponentListener(j);

		if ( in != null )
			j.restore(in);
		return j;
	}
}

class KitchenPropertyChangeListener implements PropertyChangeListener {
	public void propertyChange(PropertyChangeEvent e) {
System.out.println("KitchenPropertyChange e="+e);
	}
}

class KitchenFrame extends Frame {

	MenuItem matrixItem;
	MenuItem maticItem;
	MenuItem volumeItem;
	MenuItem tempoItem;
	MenuItem testItem;
	MenuItem boomerItem;
	MenuItem bigboomerItem;

	public KitchenFrame(String name) {
		super(name);

		MenuBar mb = new MenuBar();
		Menu m1 = new Menu("Cooklets",true);
		addit(m1,"MouseMatrix");
		// addit(m1,"MaticPlayer");
		// addit(m1,"Volume");
		// addit(m1,"Tempo");
		addit(m1,"Boomer");
		addit(m1,"PianoRoll");
		// addit(m1,"BigBoomer");
		mb.add(m1);

		setMenuBar(mb);
	}
	public void addit(Menu m, String s) {
		MenuItem mi = new MenuItem(s);
		m.add(mi);
		mi.addActionListener(new createCookletAction(s));
	}
}

class createCookletAction implements ActionListener {
	String _className;
	createCookletAction(String cn) {
		_className = cn;
	}
	public synchronized void actionPerformed(ActionEvent e) {
		Kitchen.createCooklet("com.nosuch.cooking."+_className,null);
	}
}

class CookletWindowListener extends WindowAdapter {
	Frame _f;
	Cooklet _j;
	CookletWindowListener(Frame f, Cooklet j) {
		_f = f;
		_j = j;
	}
	public void windowClosing(WindowEvent e) {
		_j.stop();
		_f.dispose();
	}
}

class TopWindowListener extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
		MidiSystem.stop();
		System.exit(0);
	}
}
