/* NoSuch MIDI - Copyright 1997 by Tim Thompson */

package com.nosuch.midi;

/**
 * MidiAction is an interface for the callbacks done by MidiFsm.
 *
 * @author Tim Thompson
 */

public class MidiAction {
	/** This is called when a note-on message is received.
	 * @param c channel (1-16)
	 * @param p pitch (0-127)
	 * @param v velocity (0-127)
	 */
	public void on(int c, int p, int v) { }

	/** This is called when a note-off message is received.
	 * @param c channel (1-16)
	 * @param p pitch (0-127)
	 * @param v velocity (0-127)
	 */
	public void off(int c, int p, int v) { }

	/** This is called when a controller message is received.
	 * @param c channel (1-16)
	 * @param n controller number (0-127)
	 * @param v controller value (0-127)
	 */
	public void controller(int c, int n, int v) { }

	/** This is called when a program change message is received.
	 * @param c channel (1-16)
	 * @param p program number (0-127)
	 */
	public void program(int c, int p) { }

	/** This is called when a pressure message is received.
	 * @param c channel (1-16)
	 * @param p pitch (0-127)
	 * @param v value (0-127)
	 */
	public void pressure(int c, int p, int v) { }

	/** This is called when a channel pressure message is received.
	 * @param c channel (1-16)
	 * @param p value (0-127)
	 */
	public void channelPressure(int c, int p) { }

	/** This is called when a pitchbend message is received.
	 * @param	c	channel (1-16)
	 * @param	msb	most significant byte (0-127)
	 * @param	lsb	least significant byte (0-127)
	 */
	public void pitchBend(int c, int msb, int lsb) { }

	/** This is called when a system exclusive message is received.
	 * @param bytes array of bytes containing message
	 */
	public void sysex(byte bytes[]) { }

	/** This is called when a song position pointer message is received.
	 */
	public void position(int b1, int b2) { }

	/** This is called when a song message is received.
	 * @param v value
	 */
	public void song(int v) { }

	/** This is called when a reset message received. */
	public void reset() { }

	/** This is called when a timing (clock) message received. */
	public void timing() { }

	/** This is called when a start message received. */
	public void start() { }

	/** This is called when a stop message received. */
	public void stop() { }

	/** This is called when a continue message received. */
	public void midiContinue() { }

	/** This is called when a tune request message received. */
	public void tune() { }

	/** This is called when an eox (end of sysex) message is received. */
	public void eox() { }
}
