/* NoSuch MIDI - Copyright 1997 by Tim Thompson */

package com.nosuch.midi;

/**
 * A MidiNoteOn is a MidiMsg that is a single isolated note-on message.
 *
 * @author	Tim Thompson
 */

public class MidiNoteOn extends MidiMsg implements Cloneable, MidiNoteable {

	private int pitch;
	private int channel;
	private int velocity;

	/**
	 * Constructor
	 * @param t time (in clicks)
	 * @param p pitch (0 to 127)
	 * @param c channel (1 to 16)
	 * @param v velocity (0 to 127)
	 */
	public MidiNoteOn(int p, long t, int c, int v) {
		super(t);
		pitch = p;
		channel = c;
		velocity = v;
	}

	/**
	 * Constructor
	 * @param p pitch (0 to 127)
	 */
	public MidiNoteOn(int p) {
		this(p,(long)0,1,64);
	}

	public Object clone() {
		return new MidiNoteOn(pitch,time,channel,velocity);
	}

	public boolean equals(MidiMsg m) {
		if ( ! super.equals(m) )
			return false;
		if ( ! ( m instanceof MidiNoteOn) )
			return false;
		MidiNoteOn mn = (MidiNoteOn)m;
		if ( mn.pitch != pitch )
			return false;
		if ( mn.channel != channel )
			return false;
		if ( mn.velocity != velocity )
			return false;
		return true;
	}

	public int getPitch() { return pitch; }
	public void setPitch(int p) { pitch = p; }

	public long getDuration() { return 0; }
	public void setDuration(long d) { }

	public int getChannel() { return channel; }
	public void setChannel(int c) { channel = c; }

	/** Get the velocity (0-127) */
	public int getVelocity() { return velocity; }

	/** Set the velocity (0-127) */
	public void setVelocity(int v) { velocity = v; }

	public byte[] getBytesOn() {
		byte[] b = new byte[3];
		b[0] = (byte)(0x90 + (channel-1));
		b[1] = (byte)pitch;
		b[2] = (byte)velocity;
		return b;
	}
	public byte[] getBytesOff() { return null; }

	public String toString() {
		String s = "MidiNoteOn(t="+Long.toString(time)
			+",p="+Integer.toString(pitch)
			+",c="+Integer.toString(channel)
			+",v="+Integer.toString(velocity)+")";
		return s;
	}
}
